#include <cstring>
#include <cstdio>
#include "dart_api.h"

Dart_NativeFunction ResolveName(Dart_Handle name, int argc, bool* auto_setup_scope);

DART_EXPORT Dart_Handle dabi_Init(Dart_Handle parent_library) {
  if (Dart_IsError(parent_library)) return parent_library;

  Dart_Handle result_code = Dart_SetNativeResolver(parent_library, ResolveName, NULL);
  if (Dart_IsError(result_code)) return result_code;

  return Dart_Null();
}

Dart_Handle HandleError(Dart_Handle handle) {
 if (Dart_IsError(handle)) Dart_PropagateError(handle);

 return handle;
}

void bind(Dart_NativeArguments arguments) {
  Dart_EnterScope();

  Dart_Handle json_map_handle = HandleError(Dart_GetNativeArgument(arguments, 0));
  Dart_Handle type_handle = HandleError(Dart_GetNativeArgument(arguments, 1));

  if (Dart_IsType(type_handle) && Dart_IsMap(json_map_handle)) {
    intptr_t length;
    Dart_Handle keys = HandleError(Dart_MapKeys(json_map_handle));
    
    HandleError(Dart_ListLength(keys, &length));

    Dart_Handle obj = HandleError(Dart_Allocate(type_handle));

    Dart_Handle key;
    for (intptr_t i = 0; i < length; i++) {
      key = HandleError(Dart_ListGetAt(keys, i));
      Dart_Handle err = Dart_SetField(obj, key, HandleError(Dart_MapGetAt(json_map_handle, key)));
      if (Dart_IsError(err)) {}
    }

    Dart_SetReturnValue(arguments, obj);
  } else {
    Dart_SetReturnValue(arguments, Dart_Null());
  }

  Dart_ExitScope();
}

Dart_NativeFunction ResolveName(Dart_Handle name, int argc, bool* auto_setup_scope) {
  if (!Dart_IsString(name)) return NULL;

  Dart_NativeFunction result = NULL;
  const char* cname;
  HandleError(Dart_StringToCString(name, &cname));

  if (strcmp("bind", cname) == 0) result = bind;

  return result;
}